# -*- coding: utf-8 -*-
#
#  test_accumulate.py
#  anytop
#
#  Created by Lars Yencken on 2012-01-27.
#  Copyright 2012 Lars Yencken. All rights reserved.
#

"""
Tests for the accumulate module.
"""

import unittest

import accumulate
import random

class FloatRangeTestCase(unittest.TestCase):
    def test_range(self):
        r = accumulate.FloatRange(0, 2, 2)
        self.assertEqual(list(r), [(0.0, 1.0), (1.0, 2.0)])
        self.assertEqual(len(r), 2)

    def test_get_bin(self):
        r = accumulate.FloatRange(0, 2, 2)
        self.assertEqual(r.get_bin(-0.1), None)
        self.assertEqual(r.get_bin(0.0), (0.0, 1.0))
        self.assertEqual(r.get_bin(0.00001), (0.0, 1.0))
        self.assertEqual(r.get_bin(0.99999), (0.0, 1.0))
        self.assertEqual(r.get_bin(1.0), (1.0, 2.0))
        self.assertEqual(r.get_bin(1.00001), (1.0, 2.0))
        self.assertEqual(r.get_bin(1.99999), (1.0, 2.0))
        self.assertEqual(r.get_bin(2.0), None)
        self.assertEqual(r.get_bin(2.1), None)

    def test_get_bin_2(self):
        frange = accumulate.FloatRange(1, 7, 6)
        self.assertEqual(list(frange), [
            (1.0, 2.0),
            (2.0, 3.0),
            (3.0, 4.0),
            (4.0, 5.0),
            (5.0, 6.0),
            (6.0, 7.0),
        ])
        self.assertEqual(frange.get_bin(0), None)
        self.assertEqual(frange.get_bin(1), (1.0, 2.0))
        self.assertEqual(frange.get_bin(2), (2.0, 3.0))
        self.assertEqual(frange.get_bin(3), (3.0, 4.0))
        self.assertEqual(frange.get_bin(4), (4.0, 5.0))
        self.assertEqual(frange.get_bin(5), (5.0, 6.0))
        self.assertEqual(frange.get_bin(6), (6.0, 7.0))
        self.assertEqual(frange.get_bin(7), None)

class NumericAccumulatorTestCase(unittest.TestCase):
    def test_dist(self):
        data = [0, 1, 1, 2, 3, 3, 3, 4, 6, 6, 6, 7, 7, 8]
        random.shuffle(data)
        frange = accumulate.FloatRange(1, 7, 6)
        assert frange.end == 7.0
        self.assertEqual(frange.get_bin(6), (6.0, 7.0))
        acc = accumulate.NumericAccumulator()
        acc.consume_all(data)
        self.assertEqual(acc, sorted(data))
        dist = acc.get_dist(frange)
        expected_dist = {
                None:       4,
                (1.0, 2.0): 2,
                (2.0, 3.0): 1,
                (3.0, 4.0): 3,
                (4.0, 5.0): 1,
                (5.0, 6.0): 0,
                (6.0, 7.0): 3,
            }
        for k in expected_dist:
            self.assertEqual(dist[k], expected_dist[k])

        assert set(dist).issubset(set(expected_dist))

class AccumulatorTestCase(unittest.TestCase):
    def test_basic(self):
        data = ['a', 'a', 'b', 'c', 'a']
        acc = accumulate.Accumulator()
        for x in data:
            acc.consume(x)

        expected_dist = {'a': 3, 'b': 1, 'c': 1}
        dist = acc.to_dist()
        for k in expected_dist:
            self.assertEqual(dist[k], expected_dist[k])
        assert set(dist).issubset(expected_dist)

class WindowAccumulatorTestCase(unittest.TestCase):
    def test_basic(self):
        acc = accumulate.WindowAccumulator(3)
        for x in ['a', 'a', 'b']:
            acc.consume(x)
        dist = acc.to_dist()
        self.assertEqual(dist['a'], 2)
        self.assertEqual(dist['b'], 1)

        acc.consume('c')
        acc.consume('b')
        dist = acc.to_dist()
        self.assertEqual(dist['b'], 2)
        self.assertEqual(dist['c'], 1)

def suite():
    return unittest.TestSuite((
            unittest.makeSuite(FloatRangeTestCase),
            unittest.makeSuite(NumericAccumulatorTestCase),
        ))

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=1).run(suite())




# -*- coding: utf-8 -*-
#
#  common.py
#  anytop
#
#  Created by Lars Yencken on 2012-01-22.
#  Copyright 2012 Lars Yencken. All rights reserved.
#

"""
Common methods for command-line operation.
"""

import curses
import re

MAX_IO_RETRIES = 2

_color_pattern = re.compile("\x1b\[[0-9]*(;[0-9]*)?m", re.UNICODE)

def init_win(win):
    curses.start_color()
    curses.use_default_colors()
    curses.curs_set(0)
    win.nodelay(1)

def robust_line_iter(istream):
    "Read lines, continuing if a blocking system call gets interrupted."
    tries_since_success = 0
    sub = _color_pattern.sub

    lines = unbuffered_lines(istream)
    while True:
        try:
            l = lines.next()
            tries_since_success = 0

            # trim and remove shell colors
            l_no_color = sub(u'', l.rstrip())

            yield l_no_color

        except StopIteration:
            break

        except IOError:
            # sometimes we can get interrupted on a blocking read
            # let's retry a few times
            tries_since_success += 1

            if tries_since_success >= MAX_IO_RETRIES:
                # something's really wrong
                break

def unbuffered_lines(istream):
    "Read from the stream using the less-buffered readline() method."
    l = istream.readline()
    while l:
        yield l
        l = istream.readline()

def get_zoom(largest, width):
    """
    Determine an appropriate zoom ratio so that the given value will fit
    within width.
    """
    scale = 0
    options = [1, 2, 3, 5]

    while True:
        for x in options:
            zoom = x * (10**scale)
            if largest / zoom <= width:
                return zoom
        scale += 1




# -*- coding: utf-8 -*-
#
#  __init__.py
#  anytop
#
#  Created by Lars Yencken on 2012-01-27.
#  Copyright 2012 Lars Yencken. All rights reserved.
#

"""
Supporting utilities for the anytop package.
"""




# -*- coding: utf-8 -*-
#
#  test_common.py
#  anytop
#
#  Created by Lars Yencken on 2012-01-27.
#  Copyright 2012 Lars Yencken. All rights reserved.
#

"""
Tests for the common utilities.
"""

import unittest

import common

class ZoomTestCase(unittest.TestCase):
    def test_zoom(self):
        self.assertEqual(common.get_zoom(0, 100), 1)
        self.assertEqual(common.get_zoom(99, 100), 1)
        self.assertEqual(common.get_zoom(100, 100), 1)
        self.assertEqual(common.get_zoom(101, 100), 2)
        self.assertEqual(common.get_zoom(201, 100), 2)
        self.assertEqual(common.get_zoom(202, 100), 3)
        self.assertEqual(common.get_zoom(303, 100), 5)
        self.assertEqual(common.get_zoom(505, 100), 10)
        self.assertEqual(common.get_zoom(1010, 100), 20)

def suite():
    return unittest.TestSuite((
            unittest.makeSuite(ZoomTestCase),
        ))

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=1).run(suite())




# -*- coding: utf-8 -*-
#
#  buffer.py
#  anytop
#
#  Created by Lars Yencken on 2012-01-27.
#  Copyright 2012 99designs. All rights reserved.
#

"""
Buffered iteration support.
"""

class BufferedIter(object):
    def __init__(self, it):
        self.it = iter(it)
        self._head = None
        self._empty = False

        self._advance()

    def __iter__(self):
        return self

    def next(self):
        if self._empty:
            raise StopIteration

        head = self._head
        self._advance()
        return head
    
    def empty(self):
        return self._empty

    def peek(self):
        if self._empty:
            raise ValueError

        return self._head

    def _advance(self):
        try:
            self._head = self.it.next()
        except StopIteration:
            self._empty = True

    def dropwhile(self, predicate):
        "Drop a prefix that matches the given predicate."
        while not self.empty() and predicate(self._head):
            self._advance()

    def takewhile(self, predicate):
        while not self.empty() and predicate(self._head):
            yield self.peek()
            self._advance()





# -*- coding: utf-8 -*-
#
#  test_buffered.py
#  anytop
#
#  Created by Lars Yencken on 2012-01-27.
#  Copyright 2012 Lars Yencken. All rights reserved.
#

"""
Tests for buffered iteration support.
"""

import unittest

from buffered import BufferedIter as BI

class BufferedIterTest(unittest.TestCase):
    def test_iteration(self):
        lbi = lambda it: list(BI(it))
        self.assertEqual(lbi([]), [])
        self.assertEqual(lbi(xrange(3)), range(3))

    def test_peek(self):
        it = BI(xrange(4))
        assert not it.empty()
        self.assertEqual(it.peek(), 0)
        self.assertEqual(it.next(), 0)
        assert not it.empty()
        self.assertEqual(it.peek(), 1)
        self.assertEqual(it.next(), 1)
        assert not it.empty()
        self.assertEqual(it.peek(), 2)
        self.assertEqual(it.next(), 2)
        assert not it.empty()
        self.assertEqual(it.peek(), 3)
        self.assertEqual(it.next(), 3)
        assert it.empty()
        self.assertRaises(ValueError, it.peek)

    def test_dropwhile(self):
        it = BI(xrange(30))
        it.dropwhile(lambda x: x < 20)
        self.assertEqual(it.peek(), 20)
        self.assertEqual(list(it), range(20, 30))

    def test_takewhile(self):
        it = BI(xrange(30))
        self.assertEqual(list(it.takewhile(lambda x: x < 5)), [0, 1, 2, 3, 4])
        self.assertEqual(it.peek(), 5)
        self.assertEqual(list(it), range(5, 30))

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=1).run(
            unittest.makeSuite(BufferedIterTest))



# -*- coding: utf-8 -*-
#
#  accumulate.py
#  anytop
#
#  Created by Lars Yencken on 2012-01-27.
#  Copyright 2012 Lars Yencken. All rights reserved.
#

"""
Accumulate input into a useful summary.
"""

from collections import defaultdict, deque
import bisect
import itertools

class FloatRange(object):
    "A floating point range representing frequency bins."
    def __init__(self, start, end, n):
        self.start = float(start)
        self.end = float(end)
        self.n = n
        self.interval = (self.end - self.start) / float(n)
        self.bin_markers = self._gen_bins()

    def _gen_bins(self):
        bins = [self.start]
        for i in xrange(1, self.n + 1):
            bins.append(self.start + i * self.interval)
        assert abs(bins[-1] - self.end) < 1e-6
        return bins
    
    def __iter__(self):
        for i in xrange(self.n):
            yield (self.bin_markers[i], self.bin_markers[i+1])
    
    def __len__(self):
        return self.n
    
    def get_bin(self, x, default=None):
        i = bisect.bisect(self.bin_markers, x)
        if i == 0:
            return default
        if i == len(self.bin_markers):
            return default
        return (self.bin_markers[i-1], self.bin_markers[i])

class NumericAccumulator(list):
    def consume(self, x):
        bisect.insort(self, x)

    def consume_all(self, xs):
        for x in xs:
            self.consume(x)

    def get_dist(self, frange):
        """
        Given a start point, and then the end points of n sequential buckets,
        return a mapping from bucket index to counts.
        """
        dist = defaultdict(int)
        for bucket_index, xs in itertools.groupby(self, key=frange.get_bin):
            xs = list(xs)
            count = sum(1 for x in xs)
            dist[bucket_index] += count

        return dist

class Accumulator(defaultdict):
    "Accumulate counts without end."
    def __init__(self):
        super(Accumulator, self).__init__(int)

    def consume(self, key):
        self[key] += 1

    def to_dist(self):
        return self

class WindowAccumulator(deque):
    "Accumulate counts within a fixed-size rolling window."
    def __init__(self, n):
        super(WindowAccumulator, self).__init__([], n)

    def consume(self, key):
        self.append(key)

    def to_dist(self):
        d = defaultdict(int)
        for t in self:
            d[t] += 1
        return d

